#include <iostream>
#include <map>
#include <vector>
#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>
#include <boost/serialization/map.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/foreach.hpp>
#include <fstream>

struct S
{
  friend class boost::serialization::access;
  std::string foo;
  int bar;

  template<class Archive>
  void serialize(Archive& ar, const unsigned int version)
  {
    ar & foo;
    ar & bar;
  }

  S()
  {}
  S(const std::string& f, const int& b)
    :foo(f),bar(b)
  {}
};

std::ostream& operator<<(std::ostream& ost, const S& s)
{
  return ost<<"("<<s.foo<<":"<<s.bar<<")";
}

class Thing
{
private:
  friend class boost::serialization::access;
  friend std::ostream& operator<<(std::ostream& ost, const Thing& t);
  typedef std::vector<S> svec_t;
  typedef std::map<std::string,svec_t> map_t;

  map_t the_map_m;
  template<class Archive>
  void serialize(Archive& ar, const unsigned int version)
  {
    ar & the_map_m;
  }
public:
  void fill_map(const std::string& prefix)
  {
    svec_t first;
    first.push_back(S("hej",0));
    first.push_back(S("wenakkam",1));
    first.push_back(S("salam",2));
    svec_t second;
    first.push_back(S("foo",17));
    first.push_back(S("bar",18));
    first.push_back(S("???",19));

    the_map_m[prefix+"hello"] = first;
    the_map_m[prefix+"foobar"] = second;
  }
  Thing()
  {}
};

std::ostream& operator<<(std::ostream& ost, const Thing& t)
{
  ost<<"The map:";
  BOOST_FOREACH(const Thing::map_t::value_type& m, t.the_map_m){
    ost<<std::endl<<m.first<<":";
    BOOST_FOREACH(const Thing::svec_t::value_type& s, m.second){
      ost<<s<<",";
    }
  }
  ost<<std::endl<<"----";
  return ost;
}

int main()
{

  {
    std::ofstream ofs("test.dat");

    Thing t;
    t.fill_map("ttttt_");
    
    Thing t2;
    t2.fill_map("22222_");

    boost::archive::text_oarchive oa(ofs);
    oa << t;
    oa << t2;
  }

  {
    std::ifstream ifs("test.dat");
    boost::archive::text_iarchive ia(ifs);

    Thing t,t2;
    
    ia >> t;
    ia >> t2;

    std::cout<<t<<std::endl;
    std::cout<<t2<<std::endl;
  }
}
